@@ -10,11 +10,12 @@
 use super::families;
 use fontdb::{FaceInfo, Source};
 pub use fontdb::{Stretch, Style, Weight};
-use log::{info, warn};
+use log::{debug, info, trace, warn};
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
 use std::borrow::Cow;
 use std::collections::hash_map::{Entry, HashMap};
+use std::fmt;
 use std::path::Path;
 
 /// How to add new aliases when others exist
@@ -29,7 +30,7 @@ pub enum AddMode {
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 enum State {
     /// Newly created. If the boolean is true, system fonts will be loaded at
-    /// init time.
+    /// initialization time.
     New(bool),
     Ready,
 }
@@ -46,7 +47,7 @@ fn to_uppercase<'a>(c: Cow<'a, str>) -> Cow<'a, str> {
 /// This database exists as a singleton, accessible through the [`fonts`]
 /// function.
 ///
-/// After initialisation font loading and alias adjustment is disabled. The
+/// After initialization font loading and alias adjustment is disabled. The
 /// reason for this is that font selection uses multiple caches and
 /// there is no mechanism for forcing fresh lookups everywhere.
 ///
@@ -54,7 +55,7 @@ fn to_uppercase<'a>(c: Cow<'a, str>) -> Cow<'a, str> {
 pub struct Database {
     state: State,
     db: fontdb::Database,
-    families_upper: HashMap<String, usize>,
+    families_upper: HashMap<String, Vec<usize>>,
     // contract: all keys and values are uppercase
     aliases: HashMap<Cow<'static, str>, Vec<Cow<'static, str>>>,
 }
@@ -147,7 +148,7 @@ impl Database {
             .get(family)
             .and_then(|list| list.iter().next())
             .map(|name| {
-                let index = *families_upper.get(name.as_ref()).unwrap();
+                let index = families_upper.get(name.as_ref()).unwrap()[0];
                 db.faces()[index].family.clone()
             })
     }
@@ -157,8 +158,8 @@ impl Database {
     /// When searching for `family`, all `aliases` will be searched too. Both
     /// the `family` parameter and all `aliases` are converted to upper case.
     ///
-    /// This method may only be used before init; if used afterwards, only a
-    /// warning will be issued.
+    /// This method may only be used before initialization; if used afterwards,
+    /// only a warning will be issued.
     pub fn add_aliases<I>(&mut self, family: Cow<'static, str>, aliases: I, mode: AddMode)
     where
         I: Iterator<Item = Cow<'static, str>>,
@@ -192,7 +193,7 @@ impl Database {
         }
     }
 
-    /// Control whether system fonts will be loaded on init
+    /// Control whether system fonts will be loaded on initialization
     ///
     /// Default value: true
     pub fn set_load_system_fonts(&mut self, load: bool) {
@@ -205,8 +206,9 @@ impl Database {
     ///
     /// Will load all font faces in case of a font collection.
     ///
-    /// This method may only be used before init; if used afterwards, only a
-    /// warning will be issued. By default, system fonts are loaded on init.
+    /// This method may only be used before initialization; if used afterwards,
+    /// only a warning will be issued. By default, system fonts are loaded on
+    /// initialization.
     pub fn load_font_data(&mut self, data: Vec<u8>) {
         if &self.state == &State::Ready {
             warn!("unable to load fonts after font DB init");
@@ -219,8 +221,8 @@ impl Database {
     ///
     /// Will load all font faces in case of a font collection.
     ///
-    /// This method may only be used before init; if used afterwards, only a
-    /// warning will be issued. By default, system fonts are loaded on init.
+    /// This method may only be used before initialization; if used afterwards, only a
+    /// warning will be issued. By default, system fonts are loaded on initialization.
     pub fn load_font_file<P: AsRef<Path>>(&mut self, path: P) -> Result<(), std::io::Error> {
         if &self.state == &State::Ready {
             warn!("unable to load fonts after font DB init");
@@ -238,8 +240,8 @@ impl Database {
     /// Unlike other `load_*` methods, this one doesn't return an error.
     /// It will simply skip malformed fonts and will print a warning into the log for each of them.
     ///
-    /// This method may only be used before init; if used afterwards, only a
-    /// warning will be issued. By default, system fonts are loaded on init.
+    /// This method may only be used before initialization; if used afterwards, only a
+    /// warning will be issued. By default, system fonts are loaded on initialization.
     pub fn load_fonts_dir<P: AsRef<Path>>(&mut self, dir: P) {
         if &self.state == &State::Ready {
             warn!("unable to load fonts after font DB init");
@@ -255,14 +257,14 @@ impl Database {
             }
             info!("Found {} fonts", self.db.len());
 
-            self.families_upper = self
-                .db
-                .faces()
-                .iter()
-                .enumerate()
-                .map(|(i, face)| (face.family.to_uppercase(), i))
-                .collect();
-            let families_upper = &self.families_upper;
+            let families_upper = &mut self.families_upper;
+            for (i, face) in self.db.faces().iter().enumerate() {
+                trace!("Discovered: {}", DisplayFaceInfo(face));
+                families_upper
+                    .entry(face.family.to_uppercase())
+                    .or_default()
+                    .push(i);
+            }
 
             for aliases in self.aliases.values_mut() {
                 // Remove aliases to missing fonts:
@@ -327,7 +329,7 @@ pub struct FontSelector<'a> {
 impl<'a> FontSelector<'a> {
     /// Synonym for default
     ///
-    /// Without further parametrisation, this will select a generic sans-serif
+    /// Without further parametrization, this will select a generic sans-serif
     /// font which should be suitable for most uses.
     #[inline]
     pub fn new() -> Self {
@@ -358,7 +360,7 @@ impl<'a> FontSelector<'a> {
     /// is used. Glyph-level fallback (missing glyph substitution) is not
     /// currently supported.
     ///
-    /// If an empty vec is passed, the default "sans-serif" font is used.
+    /// If an empty vector is passed, the default "sans-serif" font is used.
     #[inline]
     pub fn set_families(&mut self, mut names: Vec<Cow<'a, str>>) {
         for x in &mut names {
@@ -403,6 +405,7 @@ impl<'a> FontSelector<'a> {
     where
         F: FnMut(&'b Source, u32) -> Result<(), Box<dyn std::error::Error>>,
     {
+        debug!("select(): {:?}", self);
         // TODO(opt): improve, perhaps moving some computation earlier (e.g.
         // culling aliases which do not resolve fonts), and use faster alias expansion.
         let mut families: Vec<Cow<'b, str>> = self.families.clone();
@@ -431,8 +434,12 @@ impl<'a> FontSelector<'a> {
         let mut candidates = Vec::new();
         // Step 3: find any matching font faces, case-insensitively
         for family in families {
-            if let Some(index) = db.families_upper.get(family.as_ref()) {
-                candidates.push(&db.db.faces()[*index]);
+            if let Some(indices) = db.families_upper.get(family.as_ref()) {
+                for index in indices {
+                    let candidate = &db.db.faces()[*index];
+                    trace!("candidate: {}", DisplayFaceInfo(candidate));
+                    candidates.push(candidate);
+                }
             }
 
             // Step 4: if any match from a family, narrow to a single face.
@@ -586,6 +593,21 @@ impl<'a> FontSelector<'a> {
     }
 }
 
+struct DisplayFaceInfo<'a>(&'a FaceInfo);
+impl<'a> fmt::Display for DisplayFaceInfo<'a> {
+    fn fmt(&self, f: &mut fmt::Formatter) -> Result<(), fmt::Error> {
+        let path = match &*self.0.source {
+            Source::Binary(_) => None,
+            Source::File(path) => Some(path.display()),
+        };
+        write!(
+            f,
+            "family=\"{}\", source={:?},{}",
+            self.0.family, path, self.0.index
+        )
+    }
+}
+
 // See: https://serde.rs/remote-derive.html
 #[cfg(feature = "serde")]
 mod remote {
